@@ -1,7 +1,7 @@
 /* 
- * twcw - Copyright (C) 2006 by Ted Williams - WA0EIR
- * Derived mostly from qrq.c
- * High speed morse trainer by Fabian Kurz, DJ1YFK, Copyright (C) 2006, and
+ * libCW- 2.0 - Copyright (C) 2006 -> 2011 by Ted Williams - WA0EIR
+ * Derived mostly from qrq.c - High speed morse trainer
+ * by Fabian Kurz, DJ1YFK, Copyright (C) 2006, and
  * a few bits from unixcw
  * by Simon Baldwin, G0FRD.Copyright (C) 2001-2006
  * 
@@ -18,42 +18,107 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/ 
+ */ 
 
+/*
+ * include file for libCW.h
+ */
 #include "libCW.h"
 
-#define TESTMODE 0
-#define MESG 0        /* 0 = no message support, 1 =  with message support */
-#define DEBUG 0
+#include <errno.h>
+#include <stdio.h>
+#include <ctype.h>
+#include <math.h>
+#include <fcntl.h>
+#include <signal.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <pthread.h>
+#include <sys/kd.h>
+#include <sys/msg.h>
+#include <sys/ioctl.h>
+#include <linux/rtc.h>
+
+#include <pulse/simple.h>
+#include <pulse/error.h>
+
+#define TESTMODE  0
+#define MESG      0       /* 0 = no message , 1 =  with message support */
+#define DEBUG     0
+
+#define FUDGE     1.066   /* fudge factor for my lousy soundcard */
+
+#define MAX_NAME  20
+#define MAX_LETTERS 512
+#define MAX_WORD_LENGTH  100L    /* Maximum length of a word */
+#define PI 3.1415926             /* Indiana residents, feel free to */
+                                 /* change it to 3.2 */
+#define DIT '.'
+#define DAH '-'
+#define DEFAULT_WPM   20.0
+#define MIN_WPM        5.0
+#define FARNS_OFF  MIN_WPM - 1.0  /* minimum farnsworth speed */
+#define RISE 2.0
+#define FALL 2.0
+
+#define SAMPLERATE 8000
+//#define SAMPLERATE 44100
+
+#define TRUE 1
+#define FALSE !TRUE
+#define SENT_MSG 7L              /* as defined in common.h */
+
+/*
+ * CWirc stuff
+*/
+#if WITH_CWIRC == 1
+#define AUDIOBUF_SIZE 22050   /* Samples */
+struct cwirc_extension_api {
+   int semid;
+   signed short int out_audiobuf[AUDIOBUF_SIZE];
+   int out_audiobuf_start;
+   int out_audiobuf_end;
+   int in_key;
+   int pid;
+   };
+extern struct cwirc_extension_api *cwirc_api_shm;
+#endif
 
 #if MESG == 1
 extern int qid;
 #endif
 
-int doReply;                                   /* send reply msg when true */
-int key_modes = 0;                             /* no key modes selected */
+/* set some variables to their default values */
+char serdevice[MAX_NAME] = "/dev/ttyS0";       /* default serial device path */
+int  ser_fd = 0;
+char pa_name[MAX_NAME];                        /* shows up on pa vol ctrl */
+int keyMode = 0;                               /* PA, RTS, or CWIRC */
+int doReply = FALSE;                           /* send reply msg when true */
 
 int rts_on = TIOCM_RTS;                        /* for key down */
 int rts_off = 0;                               /* for key up */
-int initialized = -1;                          /* set to 1 by cw_init */
+int initialized = 0;                           /* set to 1 by cw_init */
 
-char dspdevice[20] = DEFAULT_SC_DEVICE;        /* default dsp */
-int  dsp_fd = 0;
-char serdevice[20] = DEFAULT_RTS_DEVICE;       /* default serial */
-int  ser_fd = 0;
-char mixdevice[20] = DEFAULT_MIX_DEVICE;       /* default mixer */
-int  mix_fd = 0;
-int  rtc_rate = 2048;                          /* RTC interupt rate */
+/* for PA */
+static const pa_sample_spec ss =
+   {
+      .format = PA_SAMPLE_S16LE,
+      .rate = SAMPLERATE,
+      .channels = 2
+   };
+
+   pa_simple *sp = NULL;
+
+int rtc_rate = 4096;                          /* RTC interupt rate */
 
 char *codetable[MAX_LETTERS];
 
-int  send_vol = (DEFAULT_VOL<<8)+DEFAULT_VOL;  /* sending vol level */
-int  save_vol;                                 /* original vol level */
-static int  freq = 500;                        /* freq value for SC */
+static int freq = 700;                        /* freq value for SC */
 static long samplerate = SAMPLERATE;
 
-float wpm = DEFAULT_WPM;                       /* current speed in wpm */
-float farns_wpm = DEFAULT_FARNS;               /* farnsworth speed in wpm */
+float wpm = DEFAULT_WPM;                      /* current speed to default */
+float farns_wpm = FARNS_OFF;                  /* farnsworth speed to default */
 
 float dit_time = 1200.0 / DEFAULT_WPM;
 float dah_time = 1200.0 / DEFAULT_WPM * 3;     /* dah time in milliseconds */
@@ -63,82 +128,118 @@ float ic_time = 1200 / DEFAULT_WPM;            /* intra letter time */
 float il_time = 3 * 1200 / DEFAULT_WPM;        /* inter letter time */
 float iw_time = 7 * 1200 / DEFAULT_WPM;        /* inter word time */
 
-static float rise = RISE;                      /* risetime in ms */
-static float fall = FALL;                      /* fall time in ms */
-static float rt = SAMPLERATE / 1000 * RISE;    /* normalized risetime */
-static float ft = SAMPLERATE / 1000 * FALL;    /* normalized falltime */
+float rise = RISE;                      /* risetime in ms */
+float fall = FALL;                      /* fall time in ms */
+float rt = SAMPLERATE / 1000 * RISE;    /* normalized risetime */
+float ft = SAMPLERATE / 1000 * FALL;    /* normalized falltime */
 
-static pthread_t cwthread;                     /* thread for CW output */
+static pthread_t cwthread = 0;                 /* thread for CW output */
 static pthread_attr_t cwattr;                  /* thread attributes */
 
+#if WITH_CWIRC == 1
 extern struct cwirc_extension_api *cwirc_api_shm;
 struct cwirc_extension_api *cwirc_api_shm;
+#endif
 
-/*internal use */
+/* forward declarations internal functions */
+int  cw_set_keyMode (int newmode, char *pgmname);
 void cleanup ();
 void *send_cw (void *p);
 void keySC (char *code);
 void keyRTS (char *code);
-void keyIRC (char *code);
-int  tonegen (int freq, float length);
-int  get_volume (void);
-int  set_volume (int vol);
-int  open_dsp (char *device);
-int  open_mixer (char *device);
-int  open_serial (char *device);
+int  open_serial (void);
 int  timer (float usec);
 int  close_devs(void);
+int  tonegen (int freq, float length);
+
+#if WITH_CWIRC == 1
+void keyCWIRC (char *code);
+#endif
 
 
 /*
  * cw_init function
+ * if name and ser args are present, copy them to local vars.
+ * then, set the mode to PA, RTS or CWIRC. PA needs to create a new pa stream.
+ * RTS needs to open the serial card.  CWIRC just works. Then init the
+ * character table and setup traps.  All need the thread created.
  */
-int cw_init (char *dsp, char *mix, char *ser)
+int cw_init (int newmode, char name[], char ser[])
 {
-   int i, fd, rtn;
+   int i, rtn;
 
-   /* save the dsp, mixer and serial args if not NULL */
-   /* must have both dsp and mix to use the sound card */
+   /*join the thread if already initialized */
+   if (initialized == 1)
+   {
+      pthread_join (cwthread, NULL); 
+      initialized = 0;
+   }
 
-   if (dsp != NULL && mix != NULL)
+   /* check for a name argument */
+   if (name == NULL)
    {
-      strcpy (dspdevice, dsp);
-      strcpy (mixdevice, mix);
-      mix_fd = open_mixer (mixdevice);
-      save_vol = get_volume();
-      close (mix_fd);
+      fprintf (stderr, "cw_init Warning: name argument is NULL\n");
    }
-   if (ser != NULL)
+   else
+      strncpy (pa_name, name, MAX_NAME - 1);      /* copy name to pa_name */
+
+   /* check for a serial device */
+   if (ser == NULL)
    {
-      strcpy (serdevice, ser);
-      /* open rtc */
-      if ((fd = open ("/dev/rtc", O_RDONLY)) < 0)
-      {
-         perror ("cw_init: open /dev/rtc failed");
-         return -1;
-      }
-      /* find the highest interupt rate */
-      /* if 64 is the system default rate, thats kinda slow */
-      while (ioctl(fd, RTC_IRQP_SET, rtc_rate) < 0)
-      {
-         fprintf (stderr, "cw_init RTC_IRQP_SET %d failed\n", rtc_rate);
-         rtc_rate = rtc_rate/2;
-      }
-#if DEBUG == 1
-      fprintf (stderr, "RTC rate = %d\n", rtc_rate);
-#endif
-      close (fd);
+      fprintf (stderr, "cw_init Warning: serial device argument is NULL\n");
    }
+   else
+      strncpy (serdevice, ser, MAX_NAME - 1) ;    /* copy ser to serdevice */
+
+   /* set newmode to PA or RTS */
+   switch (newmode)
+   {
+      case PA:
+         #if DEBUG == 1
+         fprintf (stderr, "cw_init to PA mode\n");
+         #endif
+         
+         if (cw_set_keyMode (PA, name) != PA)  /* set keyMode to PA */
+         {
+            return 0;                          /* set keyMode failed */
+         }
+         break;
+
+      case RTS:
+         #if DEBUG == 1
+         fprintf (stderr, "cw_init to RTS mode\n");
+         #endif
+
+         if ((cw_set_keyMode (RTS, ser)) == RTS)
+         {
+            strncpy (serdevice, ser, MAX_NAME - 1); 
+         }
+         break;
+
+      #if WITH_CWIRC == 1
+      case CWIRC:
+         #if DEBUG == 1
+         fprintf (stderr, "USING CWIRC\n");
+         #endif
+         keyMode = CWIRC;
+         break;
+      #endif
+
+      default:   /* invalid mode */
+         fprintf (stderr, "cw_init error: invalid mode %d\n", newmode);
+         exit (1);
+   }
+
 
   /* Initialize the codetable array */
    for (i = 0; i < MAX_LETTERS; i++)
       codetable[i] = " ";
 
-   /* Pro Signs and Special */   
-   codetable[1]         = "......"; /* F1 = Error */
+   /* Pro Signs and Special */        /* Shift/Alt  */
+   codetable[1]         = "......";   /* F1 = Error */
    codetable[2]         = ".-.-.";    /* F2 = <AR>  */
-   codetable[3]         = "...-.-";   /* F3 = <SK>  */
-   codetable[4]         = "-.--.";    /* F4 = <KN>  */
+   codetable[3]         = "-.--.";    /* F3 = <KN>  */
+   codetable[4]         = "...-.-";   /* F4 = <SK>  */
    codetable[5]         = ".-...";    /* F5 = WAIT  */
    codetable[6]         = "-...-.-";  /* F6 = BREAK */
 
@@ -229,17 +330,15 @@ int cw_init (char *dsp, char *mix, char *ser)
    signal (SIGQUIT, (void *)cleanup);
    signal (SIGSEGV, (void *)cleanup);
 
-   /* all done so, set initialize flag */
-   initialized = 1;
-
-   /* nothing to join right now just so go for it */
+   /* create the cwthread and set initialized */
    rtn = pthread_create (&cwthread, NULL, &send_cw, (void *) "");
    if (rtn < 0)
    {
-      fprintf (stdout, "cw_send_str: pthread_create error\n");
-      return -1;
+      fprintf (stdout, "cw_init: pthread_create error\n");
+      return 0;
    }
-   return 0;
+   initialized = 1;
+   return 1;
 }
 
 
@@ -259,51 +358,43 @@ void cleanup (void)
    {
       if (ioctl (ser_fd, TIOCMSET, &rts_off) == -1)
       {
-         perror("cleanup: RTS Clear failed");
+         perror("cleanup: RTS key up failed");
       }
-    }
-   fprintf (stderr, "Thread killed\n");
+   }
+
    pthread_kill (cwthread, 9);
+   fprintf (stderr, "Thread killed\n");
+   /* close pulseaudio */
+   pa_simple_free (sp);
    exit (73);
 }
 
 
 /*
- * close_devs - close opened devices
+ * close_devs - closes ttyS after making sure the key is up.
  */
 int close_devs (void)
 {
-   int rtn = 0;
-
-   /* close used devices */
-   if (dsp_fd != 0)                      /* if dsp_fd is opened */
-   { 
-      close (dsp_fd);                    /* close dsp device */
-      dsp_fd = 0;
-      set_volume (save_vol);             /* set saved vol level */
-      rtn = 1;
-   }
 
    if (ser_fd != 0)                      /* if ser_fd is opened */
    {
       /* make sure the key is up */
       if (ioctl (ser_fd, TIOCMSET, &rts_off) < 0)
       {
-         perror ("key_other_stuff RTS Clear failed");
-         return (-1);
+         perror ("close_devs: RTS key up failed");
+         return (0);
       }
       close (ser_fd);                    /* close ser device */
       ser_fd = 0;
-      rtn = 1;
    }
-   return (rtn);
+   return (1);
 }
 
 
 /*
  * cw_send_str
- * sends a string of characters by starting the send_str thread
- * Called by other cw_send_* functions.
+ * joins the cwthread to send a string of characters
+ * called by all other cw_send_* functions.
  */
 int cw_send_str (char *str)
 {
@@ -311,22 +402,23 @@ int cw_send_str (char *str)
    static char s[MAX_WORD_LENGTH];
 
    /* cw_init must be called first */
-   if (initialized == -1)
+   if (initialized == 0)
    {
       fprintf (stderr, "cw_send_str: cw_init must be called first.\n");
-      return -1;
+      return 0;
    }
 
    /* wait for any current string */
-   pthread_join(cwthread, NULL);
-   strcpy (s, str);                                 /* save the string */
+   pthread_join (cwthread, NULL);
+
+   strcpy (s, str);                 /* save the string */
    rtn = pthread_create (&cwthread, NULL, &send_cw, (void *) s);
    if (rtn < 0)
    {
       fprintf (stdout, "cw_send_str: pthread_create error\n");
-      return -1;
+      return 0;
    }
-   return 0;
+   return 1;
 }
 
 
@@ -337,12 +429,12 @@ int cw_send_str (char *str)
  */
 int cw_send_word (char *str)
 {
-   static char new_str[MAX_WORD_LENGTH+1];
+   static char new_str[MAX_WORD_LENGTH +1 ];
 
    strcpy (new_str, str);
    strcat (new_str, " ");
    cw_send_str (new_str);
-   return (0);
+   return (1);
 }
 
 
@@ -358,7 +450,7 @@ int cw_send_char (char ch)
    str[0] = ch;
    str[1] = '\0';
    cw_send_str (str);
-   return (0);
+   return (1);
 }
 
 
@@ -372,45 +464,57 @@ void *send_cw (void *p)
    char *pt;
    char *code;
 
-#if MESG == 1
+   #if MESG == 1
    struct tag1a
    {
       long type;
       union
-         {
-            char word[MAX_WORD_LENGTH];
-            int  val;
-         } data;
+      {
+         char word[MAX_WORD_LENGTH];
+         int  val;
+      } data;
    } reply;
-#endif
+   #endif
 
-   /*
-    * open needed devices
-    */
-   if ((key_modes & SC) == SC)
-   { 
-      dsp_fd = open_dsp (dspdevice);                /* open sound card */
-   }
-   if ((key_modes & RTS) == RTS)                    /* if RTS mode */
+   if (strlen ((char *)p) == 0)     /* nothing to send */
    {
-      ser_fd = open_serial (serdevice);             /* open serdevice */
+      pthread_exit (0);
+   }
+
+   /* open the serial device if needed */
+   if (keyMode == RTS)
+   {
+      ser_fd = open_serial ();
+      if (ser_fd < 0)
+      {
+         fprintf (stderr, "send_cw: open %s failed\n", serdevice);
+         pthread_exit (0);
+      }
    }
 
    pt = (char *) p;
-   /* now run thru the string, calling keySC or keyTimer or keyIRC */
+   /* now run thru the string, and call keySC, keyTimer or keyCWIRC */
    while (pt[i] != '\0')
    {
       /* get the ./- representation for the character */
       code = codetable[(int)toupper((int)pt[i])];
+
       /* key devices */
-      if ((key_modes & SC) == SC)
+      if ((keyMode & PA) == PA)
+      {
          keySC (code);
-
-      if ((key_modes & RTS) == RTS)
+      }
+      if ((keyMode & RTS) == RTS)
+      {
          keyRTS(code);
+      }
 
-      if ((key_modes & IRC) == IRC)
-         keyIRC(code);
+#if WITH_CWIRC == 1
+      if (keyMode == CWIRC)
+      {
+         keyCWIRC(code);
+      }
+#endif
 
 #if MESG == 1
       /* if doReply is true and MESG is 1, we need to send a reply message */
@@ -418,11 +522,11 @@ void *send_cw (void *p)
       {
          reply.type = SENT_MSG;
          reply.data.word[0] = pt[i];
-         reply.data.word[1] = (char) NULL;
+         reply.data.word[1] = '\0';
          if (msgsnd (qid, (struct msgbuf *)&reply,
                      sizeof(reply.data), 0) == -1)
          {
-            perror ("keyer.c doWord: msgsnd SENT_MSG failed");
+            perror ("send_cw: msgsnd SENT_MSG failed");
             exit (1);
          }
       }
@@ -430,7 +534,7 @@ void *send_cw (void *p)
       i++;
    }
    close_devs ();
-   pthread_exit(0);
+   pthread_exit (0);
 }
 
 
@@ -446,8 +550,8 @@ void keySC (char *code)
    while (code[i] != '\0')
    {   
       if (code[i] == ' ')                /* do a space or non-valid char */
-      {                                  /* this assumes that the that */
-         tonegen (0, iw_time - il_time); /* a previous character has done */
+      {                                  /* this assumes that the */
+         tonegen (0, iw_time - il_time); /* previous character has */
          return;                         /* done the il_time already */
       }
 
@@ -455,16 +559,21 @@ void keySC (char *code)
       {
          t = dit_time;
       }
-      else
+      else if (code[i] == DAH)
       {
          t = dah_time;
       }
+      else
+      {
+         fprintf (stderr, "keySC: Program error - not a DIT or DAH\n");
+         exit (1);
+      }
 
-      set_volume (send_vol);             /* set send vol level */
       tonegen (freq, t);                 /* and send the dit/dah */
 
+      /* do ic_time if there are more elements in this letter */
       i++;
-      if (strlen (code) > i)             /* if more ./- to do */
+      if (code[i] != '\0')               /* if more ./- to do */
       {                                  /* for this character */ 
          tonegen (0, ic_time);           /* add intra character time */
       }
@@ -483,6 +592,11 @@ void keyRTS (char *code)
    int i = 0;
    float t;
 
+   if (ser_fd == 0)  /* if no serial device */
+   {
+      return;
+   }
+
    while (code[i] != '\0')
    {   
       if (code[i] == ' ')                      /* do a space or non-valid ch */
@@ -492,38 +606,41 @@ void keyRTS (char *code)
       }
 
       if (code[i] == DIT)
+      {
          t = dit_time;
+      }
       else
+      {
          t = dah_time;
+      }
 
       if (ioctl (ser_fd, TIOCMSET, &rts_on) < 0)    /* key down */
       {
-         perror("key_other_stuff: RTS Set failed");
-         exit (1);
+         perror("keyRTS: RTS key down failed");
       }
 
       timer (t);
 
       if (ioctl (ser_fd, TIOCMSET, &rts_off) < 0)   /* RTS key up */
       {
-            perror ("key_other_stuff RTS Clear failed");
+         perror ("keyRTS: RTS key up failed");
       }
-
       i++;
       if (strlen (code) > i)                   /* if more ./- to do */
       {                                        /* for this character */ 
          timer (ic_time);
       }
    }
-      timer (il_time);
+   timer (il_time);
 }
 
 
 /*
- * keyIRC
- * uses the morse representation to key the cwIRC
+ * keyCWIRC
+ * toggles cwirc_api_shm->in_key to key cwirc
  */
-void keyIRC (char *code)
+#if WITH_CWIRC == 1
+void keyCWIRC (char *code)
 {
    int i = 0;
    float t;
@@ -555,26 +672,29 @@ void keyIRC (char *code)
    timer (il_time);
    return;
 }
+#endif
 
 
 /*
  * tonegen
- * This function was taken form qrq.  Copyright (C) 2006  Fabian Kurz, DJ1YFK
+ * This function was mostly taken from qrq.
+ * Copyright (C) 2006  Fabian Kurz, DJ1YFK.  Thanks, Fabian.
  * Generates a sinus tone of frequency 'freq' and length 'len'
- * based on 'samplerate', 'rise' (rise time), 'fall' (fall time)
- * And no sound card POPS! Thanks, Fabian.
+ * based on 'samplerate', 'rt' (rise time), 'ft' (fall time)
  */
 int tonegen (int freq, float len)
 {
-   int x;
-   int out;
-   double val;
+   int x, rtn;
+   int buf[20000];   //plenty big for a 8wpm dah
+   float val;
+   int error;
 
    /* convert len from milliseconds to samples, determine rise/fall time */
-   len = (int) (samplerate * (len / 1000.0));
+   len = samplerate * len / 1000.0;
+
    for (x=0; x < len-1; x++)
    {
-      val = sin(2*PI*freq*x/samplerate);
+      val = 1 + sin(2*PI*freq*x/samplerate);
       if (x < rt)
       {
          val *= sin(PI*x/(2.0*rt));                 /* rising edge */
@@ -584,9 +704,17 @@ int tonegen (int freq, float len)
       {
          val *= sin(2*PI*(x-(len-ft)+ft)/(4*ft));   /* falling edge */
       }
-      out = (int) (val * 32500.0);
-      out = out + (out<<16);                        /* add second channel */
-      write(dsp_fd, &out, sizeof(out));
+      buf[x] = val*8000;
+      buf[x] = buf[x] + (buf[x]<<16);           /* add second channel */
+   }
+
+   /* now write the buffer */
+   rtn = pa_simple_write (sp, buf, 4*x, &error); 
+   if (rtn == -1)
+   {
+      fprintf (stderr, "pa_simple_write failed - error=%d: %s\n",
+               error, pa_strerror(error));
+      return -1;
    }
    return 0;
 }
@@ -629,7 +757,7 @@ int timer (float msec)
       if (read (fd, &rtc_data, sizeof(rtc_data)) < 0)
       {
          perror ("timer: RTC read");
-         exit (errno);
+         exit (1);
       }
       i = i + (rtc_data >> 8);
    }
@@ -637,7 +765,7 @@ int timer (float msec)
    if (ioctl (fd, RTC_PIE_OFF, 0) < 0)
    {
       perror ("timer: PIE_OFF");
-      exit (errno);
+      exit (1);
    }
    close (fd);
    return 0;
@@ -645,96 +773,130 @@ int timer (float msec)
 
 
 /*
- * cw_get_send_volume
+ * cw_set_doReply - sets/clears reply mode 
+ * set doReply to TRUE/FALSE
+//TJW join thread?
  */
-int cw_get_send_volume (void)
+int  cw_set_doReply (int new_doReply)
 {
-   return (send_vol);
-}
-
-
-/*
- * cw_set_send_volume
- * sets the cw volume level (send_vol)
- */
-int cw_set_send_volume (int new_volume)
-{
-   /* don't want to wait for thread here */
-   send_vol = (new_volume << 8) + new_volume;
-   return 0;
-}
-
-
-/*
- * set doReply - 
- */
-int  cw_set_reply_msg (int new_reply)
-{
-   doReply = new_reply;
+   doReply = new_doReply;
    return (doReply);
 }
 
 
 /*
- * get doReply - 
+ * get doReply - returns current reply mode
+//TJW join thread
  */
-int  cw_get_reply_msg (void)
+int  cw_get_doReply (void)
 {
    return (doReply);
 }
 
 
 /*
- * cw_get_key_modes
+ *cw_get_serdev - returns the current key mode
+//TJW join str?
  */
-int cw_get_key_modes (void)
+char *cw_get_serdev (void)
 {
-   return (key_modes);
-}
-
-
-/*
- * cw_set_key_modes (int modes)
- * turns on key modes specified by modes.  Use IRC, SC and RTS for modes.
- */
-int cw_set_key_modes (int modes)
-{
-   if (key_modes == 4)                /*if IRC mode, don't change it */
-      return 0;
-
-   if ((modes & IRC) == IRC)          /* IRC must be exclusive */
+   if (serdevice != NULL)
    {
-      key_modes = IRC;
+      return (serdevice);
    }
    else
    {
-      if ((modes & SC) == SC)
-      {
-         key_modes = modes;
-      }
-      if ((modes & RTS) == RTS)
-      {
-         key_modes = modes;
-      }
+      return (NULL);
    }
-   return 0;
 }
 
 
 /*
- * cw_clear_key_modes (int modes)
- * Modes specifies the key modes to turn off.  Use SC and RTS for
- * mode. They may be or'd together.
+ * cw_get_keyMode
+//TJW join thread
  */
-int cw_clear_key_modes (int modes)
+int cw_get_keyMode (void)
 {
-   key_modes = key_modes & (~modes & 0x7);
-   return 0;
+   return (keyMode);
+}
+
+
+/*
+ * cw_set_keyMode (int newmode, char *name)
+ * Use PA, RTS, or IRC for newmode.  Returns keyMode, if it works
+ */
+int cw_set_keyMode (int newmode, char *name)
+{  
+   int fd, error;
+
+#if WITH_CWIRC == 1
+   /* if current mode is CWIRC, you're stuck there. just return */
+   if (newmode == CWIRC)
+   {
+      return keyMode;
+   }
+#endif
+
+   /* if keyMode is already correct, just return */
+   if (newmode == keyMode)
+   {
+      return keyMode;
+   }
+
+   /* if initialized, wait for the current thread */
+   if (initialized == 1)
+   {
+      //pthread_join (cwthread, NULL);
+   }
+
+   /* setup the mode */
+   switch (newmode)
+   {
+      case PA:
+         /* check for a soundcard pointer */
+         if (sp == NULL)          /* don't create a sp if we have one */
+         {
+            sp = pa_simple_new (NULL, pa_name, PA_STREAM_PLAYBACK, NULL,
+                                name, &ss, NULL, NULL, &error);
+            if (sp == NULL)       /* if NULL now, we have an error */
+            {
+               perror ("cw_set_keymode: pa_simple_new failed");
+               return 0;
+            }
+         }
+         break;
+
+      case RTS:
+         /* open rtc and find the highest interupt rate */
+         rtc_rate = 4096;
+         if ((fd = open ("/dev/rtc", O_RDONLY)) < 0)
+         {
+            perror ("cw_init: open /dev/rtc failed");
+            exit (1);
+         }
+         /* find the highest interupt rate */
+         /* if 64 is the system default rate, that's kinda slow */
+         while (ioctl(fd, RTC_IRQP_SET, rtc_rate) < 0)
+         {
+            fprintf (stderr, "cw_init RTC_IRQP_SET %d failed\n", rtc_rate);
+            rtc_rate = rtc_rate/2;
+         }
+         fprintf (stderr, "RTC rate = %d\n", rtc_rate);
+         close (fd);   /* close the rtc */
+         break;
+
+      default:
+         fprintf (stderr, "cw_set_keyMode: invalid mode %d\n", newmode);
+         return 0;       /* oops!  invalid mode */
+   }
+   keyMode = newmode;
+   return keyMode;
 }
 
 
 /* 
- * cw_get_wpm
+ * cw_get_wpm - returns current wpm
+//TJW join thread?
  */
 int cw_get_wpm (void)
 {
@@ -753,19 +915,19 @@ int cw_set_wpm (int new_wpm)
    float wpm_time;
 
    /* cw_init must be called first */
-   if (initialized == -1)
+//TJW is this true?
+   if (initialized == 0)
    {
       fprintf (stderr, "cw_set_wpm: cw_init must be called first.\n");
       return -1;
    }
    /*
-    * we don't want to change speed in the middle of a word
-    * so wait for the thread if it is running.
+    * we don't want to change speed while sending somethingm
+    * so wait for the thread
     */
-    pthread_join(cwthread, NULL);
+   pthread_join (cwthread, NULL);
    /*
-    * save the new wpm values if they are >= 5 - that way this function can
-    * be used to set only one argument by setting the other argument to 0.
+    * save the new wpm value if it is >= 5
     */
    if (new_wpm >= 5)
    {
@@ -773,13 +935,21 @@ int cw_set_wpm (int new_wpm)
    }
    /*
     * speed is in wpm so we have to calculate the dot-length in
-    * milliseconds using the well-known formula  dotlength= 1200 / wpm)
+    * milliseconds using the well-known formula -> dotlength= 1200 / wpm
     */
    dit_time = 1200 / wpm;
    dah_time = 3 * dit_time;
+
+   /* fudge it for a lousy soundcard */
+   if (keyMode == PA)
+   {
+      dit_time = dit_time * FUDGE;
+      dah_time = dah_time * FUDGE;
+   }
+
    ic_time  = dit_time;
 
-   if ((farns_wpm < wpm) && (farns_wpm > 4))
+   if ((farns_wpm <= wpm) && (farns_wpm > FARNS_OFF))
    { /* do farnsworth timing */
       farns_time = 1200 / farns_wpm * 50.0; /* time to send a "word" @ fwpm */ 
       wpm_time = 31 * dit_time;             /* PARIS minus iw_time @ wpm */
@@ -791,41 +961,42 @@ int cw_set_wpm (int new_wpm)
       il_time = 3 * dit_time;
       iw_time = 7 * dit_time;
    }
-   /* rise == risetime in milliseconds, we need nr. of samples (rt) */
+   /* rc and ft are the rise and fall times in milliseconds */
    rt = (samplerate / 1000.0) * rise;
    ft = (samplerate / 1000.0) * fall;
 
-#if DEBUG == 1
-   fprintf (stderr, "BASIC dit=%4.1f dah=%4.1f il=%4.1f iw=%4.1f\n",
-           dit_time, dah_time, il_time, iw_time);
-#endif
-   return 0;
+   #if DEBUG == 1
+   fprintf (stderr, "libCW dit=%4.1f dah=%4.1f il=%4.1f iw=%4.1f rt/ft=%4.1f\n",
+           dit_time, dah_time, il_time, iw_time, rt);
+   #endif
+   return wpm;
 }
 
 
 /*
- * cw_set_freq
- * sets the frequency of the audio out
+ * cw_set_freq - sets the frequency of the audio out
+//TJW join thread
  */
 int cw_set_freq (int new_freq)
 {
    freq = new_freq;
-   return 0;
+   return freq;
 }
    
 
 /*
- * cw_get_freq
- * gets the frequency of the audio out
+ * cw_get_freq - gets the frequency of the audio out
+//TJW join thread
  */
 int cw_get_freq (void)
 {
-   return (freq);
+   return freq;
 }
 
 
-/* cw_get_fwpm 
- * returns current farnsworth wpm
+/* 
+ * cw_get_fwpm - returns current farnsworth wpm
+//TJW join thread
  */
 int cw_get_fwpm (void)
 {
@@ -835,262 +1006,155 @@ int cw_get_fwpm (void)
 
 /*
  * cw_set_fwpm
- * set farns_wpm.  Calculate the farnsworth times for  il_time and
+ * set farns_wpm.  Calculate the farnsworth times for il_time and
  * iw_time for farnsworth.
  */
 int cw_set_fwpm (int new_fwpm)
 {
    float wpm_time;
 
-   farns_wpm = new_fwpm;
+   if (initialized == 0)
+   {
+      fprintf (stderr, "cw_set_fwpm: cw_init must be called first.\n");
+      return -1;
+   }
+
    /*
-    * we don't want to change speed in the middle of a word
+    * we don't want to change speed while sending something
     * so wait for the thread if it is running.
     */
-   pthread_join(cwthread, NULL);
+   pthread_join (cwthread, NULL);
 
-   if ((farns_wpm < wpm) && farns_wpm > FARNS_OFF)
+   if ((new_fwpm < wpm) && (new_fwpm > FARNS_OFF))
    { /* do farnsworth timing */
+      farns_wpm = new_fwpm;
       farns_time = 1200 / farns_wpm * 50.0; /* time to send a "word" @ fwpm */ 
       wpm_time = 31 * dit_time;             /* PARIS minus iw_time @ wpm */
       
       il_time = 3.0/19.0 * (farns_time - wpm_time);
       iw_time = 7.0/19.0 * (farns_time - wpm_time);
    }
-#if DEBUG == 1
+   else
+   {
+      il_time = 3 * dit_time;
+      iw_time = 7 * dit_time;
+   }
+   rt = (samplerate / 1000.0) * rise;
+   ft = (samplerate / 1000.0) * fall;
+
+   #if DEBUG == 1
    fprintf (stderr, "FARNS dit=%4.1f dah=%4.1f il=%4.1f iw=%4.1f\n",
             dit_time, dit_time*3, il_time, iw_time);
-#endif
-   return 0;
-}
-
-
-/*
- * open_dsp - derived from qrq - Copyright (C) 2006  Fabian Kurz, DJ1YFK
- */
-int open_dsp (char *device)
-{
-   int tmp;
-   int fd;
-   int rtn;
-
-   if (dsp_fd != 0)                            /* if already open */
-      return (dsp_fd);
-
-   if ((fd = open(device, O_WRONLY, 0)) < 0)
-   {
-      perror("open_dsp failed");
-      exit(1);
-   }
-
-   tmp = AFMT_S16_NE; 
-   if (ioctl(fd, SNDCTL_DSP_SETFMT, &tmp)==-1)
-   {
-      perror("SNDCTL_DSP_SETFMT");
-      exit(1);
-   }
-
-   if (tmp != AFMT_S16_NE)
-   {
-      fprintf(stderr, "Cannot switch to AFMT_S16_NE\n");
-      exit(1);
-   }
-  
-   tmp = 2;	/* 2 channels, stereo */
-   if (ioctl(fd, SNDCTL_DSP_CHANNELS, &tmp)==-1)
-   {
-      perror("SNDCTL_DSP_CHANNELS");
-      exit(1);
-   }
-
-   if (tmp != 2)
-   {
-      fprintf(stderr, "No stereo mode possible :(.\n");
-      exit(1);
-   }
-
-   if ((rtn = ioctl(fd, SNDCTL_DSP_SPEED, &samplerate))==-1)
-   {
-      perror("SNDCTL_DSP_SPEED");
-      exit(1);
-   }
-   return fd;
-}
-
-
-/*
- * open_mixer
- */
-int open_mixer (char *mixdev)
-{
-   int fd;
-
-   if ((fd = open(mixdev, O_RDWR, 0)) < 0)
-   {
-      perror("open_mixer failed");
-   }
-   return fd;   
+   fprintf (stderr, "SAMPLERATE = %ld\n", samplerate);
+   #endif
+   return farns_wpm;
 }
 
 
 /*
  * open_serial
+ * open the serial device and return fd. 
  */
-int open_serial (char serdev[])
+int open_serial ()
 {
    int fd = 0;
 
-   if (ser_fd != 0)         /* already opened */
+   if (ser_fd != 0)         /* already open ?*/
+   {
+      /* yes, so just make sure the key is up and return ser_fd */
+      ioctl (fd, TIOCMSET, &rts_off);
       return (ser_fd);
-
-   if ((fd = open(serdev, O_RDWR, 0)) < 0)
-   {
-      perror("open_serial;");
-   }
-   ioctl (fd, TIOCMSET, &rts_off);
-   return fd;   
-}
-
-
-/*
- * get_volume -  This function tries to apply the PCM volume read
- * ioctl() to the main soundcard device first.  If this fails, it retries
- * using the PCM channel of the given mixer device, if it has one.
- */
-int get_volume (void)
-{
-   int vol, device_mask;
-   int fd;
-   /* Try to use the main /dev/dsp device for ioctls first. */
-   if ((fd = open_dsp (dspdevice)) < 0)
-   {
-      perror ("get_volume: open dsp failed");
-      return -1;
    }
 
-   if (ioctl (dsp_fd, MIXER_READ (SOUND_MIXER_PCM), &vol) == 0)
+   if ((fd = open(serdevice, O_RDWR, 0)) < 0)
    {
-      close (fd);
-      return vol;
-   }
-   close (fd);
-
-   /* Volume not found so try the mixer PCM channel volume instead. */
-   if ((fd = open_mixer (mixdevice)) < 0)
-   {
-      perror ("get_volume: open mixer failed");
-      return -1;
-   }
-
-   if (ioctl (fd, SOUND_MIXER_READ_DEVMASK, &device_mask) == -1)
-   {
-      perror ("get_volume: read devmask failed");
-      close (fd);
-      return -1;
-   }
-
-   if (device_mask & SOUND_MASK_PCM)
-   {
-      /* Read the PCM volume from the mixer. */
-      if (ioctl (fd, MIXER_READ (SOUND_MIXER_PCM), &vol) == -1)
-      {
-         perror ("get_volume: read PCM level failed");
-         close (fd);
-         return -1;
-      }
-      close (fd);
-      return vol;
+      perror("open_serial: can't open the serial device");
    }
    else
    {
-      if (device_mask & SOUND_MASK_VOLUME)
+      /* make sure the key is up */
+      if (fd > 0)
       {
-         /* If in extreme difficulty, try main volume. */
-         if (ioctl (fd, MIXER_READ (SOUND_MIXER_VOLUME), &vol) == -1)
-         {
-            perror ("get_volume  get main volume failed");
-            close (fd);
-            return -1;
-         }
-          close (fd);
-          return vol;
+         ioctl (fd, TIOCMSET, &rts_off);
       }
    }
-   close (fd);
-   return -1;
-}
-
-
-/*
- * set_volume - used internally to set pcm volume level
- */
-int set_volume (int vol)
-{
-   int fd;
-
-   if ((fd = open_mixer (mixdevice)) < 0)
-   {
-      perror ("set_volume-open");
-   }
-
-   if (ioctl (fd, MIXER_WRITE (SOUND_MIXER_PCM), &vol) < 0)
-   {
-      perror ("set_volume-write");
-   }
-   close (fd);
-   return 0;
+   return fd;   /* return fd or error */
 }
 
 
 #if TESTMODE == 1
+/*
+ * main function for testing standalone
+ */
 int main (int argc, char *argv[])
 {
    int i;
    char d;
-   int modes = SC;
+   char pgmName[] = "test mode";
+   char newdev[] = "/dev/ttyS0";
 
-   /* initialize lib */
-   //cw_init ("/dev/dsp", "/dev/mixer", "/dev/ttyS0");         //SoundBlaster
-   cw_init ("/dev/dsp1", "/dev/mixer1", "/dev/ttyS0");   //INTEL
+   /* pick a key mode */
+   //int mode = RTS;
+   int mode = PA;
+   //int mode = CWIRC;
 
-   /* do some other funcs */
-   cw_set_reply_msg (FALSE);
-   cw_set_wpm (20);                                    /* set wpm */
-   cw_set_fwpm (4);                                    /* set wpm */
-   cw_set_key_modes (modes);
-   cw_set_send_volume (32);                            /* set send vol */
+   /* initialize libCW */
+   cw_init (mode, pgmName, newdev);
+   //cw_init (mode, pgmName, NULL);
 
-   printf ("Enter to start: ");
-   scanf (&d);
+   /* set some other setup functions */
+   cw_set_doReply (FALSE);
+   cw_set_wpm (20);                      /* set wpm */
+   cw_set_fwpm (20);                     /* set fwpm */
 
-#if 0
-   for (i=0; i<20; i++)
+   printf ("Enter to start: ");          /* hit rtn when ready */
+   d = getchar ();
+
+#if 1
+   /* send paris as 20 strings */
+   for (i=0; i<10; i++)
    {
-      cw_send_str ("paris ");     /* send "paris" */
+      cw_send_str ("paris ");            /* send "paris" */
+
+      #if 0
+      if (i % 2 == 0)                    /* altername between PA and RTS */
+      {       /* odd ones */
+         cw_set_fwpm (10);
+      }
+      else
+      {       /* even ones */
+         cw_set_fwpm (FARNS_OFF);
+      }
+      #endif
    }
 #endif
 
 #if 0
+   /* send paris as 20 words */
    for (i=0; i<20; i++)
    {
       cw_send_word ("paris");
    }
 #endif
 
-#if 1
-   for (i=0; i<20; i++)
+#if 0
+   /* send paris as chars - twcw method */
+   for (i=0; i<4; i++)
    {
       cw_send_char ('p');
       cw_send_char ('a');
       cw_send_char ('r');
+      // cw_set_keyMode(RTS, pgmName);
       cw_send_char ('i');
       cw_send_char ('s');
       cw_send_char (' ');
+      //cw_set_fwpm (i+5);
    }
 #endif
 
+   /* beep with an 'e' when all done */
    cw_send_str ("e");
-   pthread_join(cwthread, NULL);       /* wait for it before we terminate */
+   pthread_join (cwthread, NULL);       /* wait for it before we terminate */
    return 0;
 }
 #endif   /* END TESTMODE */
