@@ -1,6 +1,6 @@
 /*
  * twclock:  A world clock implemented with openMotif widgets
- * Copyright (C) 1997 - 2007 Ted Williams WA0EIR
+ * Copyright (C) 1997 - 2011 Ted Williams WA0EIR
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,27 +17,25 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge. MA 02139, USA.
  * See the COPYING file in this directory.
  *
- * Versions: 2.7 -  Aug 2007
+ * Versions: 3.1 -  Apr 2011
  */
 
 /*
  * TWCLOCK MAIN 
  */
-
 #include "twclock.h"
 
-#ifdef MAKE_ICON
-#include "icons/twclock.xpm"
-#endif
-
-int main (int argc, char *argv[])
-{
-#ifdef MAKE_ICON 
+#if (MAKE_ICON == 1)  && (HAVE_X11_XPM_H == 1) && (HAVE_LIBXPM == 1)
+   #include <X11/xpm.h>
+   #include "icons/twclock.xpm"
    Pixmap pixmap, mask;
    XpmAttributes pix_attributes;
    XpmColorSymbol transparentColor[1] = {{NULL, "none", 0}};
 #endif
 
+
+int main (int argc, char *argv[])
+{
    XtResource appRes_desc[] =
    {
       {
@@ -47,7 +45,7 @@ int main (int argc, char *argv[])
       sizeof (int),                 /* size of target type */
       XtOffsetOf (AppRes, tzone),   /* Offset into struct */
       XtRImmediate,                 /* Default data type */
-      (XtPointer) 0                 /* Pointer to default value */
+      (XtPointer) -1                /* Defaults to -1 if Twclock not found */
       },
 
       {
@@ -57,7 +55,7 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, blink),
       XtRImmediate,
-      (XtPointer)1
+      (XtPointer) 1
       },
 
       {
@@ -67,7 +65,7 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, beep),
       XtRImmediate,
-      (XtPointer)1
+      (XtPointer) 1
       },
 
       {
@@ -77,20 +75,29 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, cwID),
       XtRImmediate,
-      (XtPointer)0
+      (XtPointer) 0
       },
 
       {
       XmNcwStr,                     /* Same as above */
       XmCCwStr,
       XtRString,
-      100,
-      //TJW sizeof (String),
+      sizeof (String),
       XtOffsetOf (AppRes, cwStr),
       XtRImmediate,
       (XtPointer) "n0call"
       },
 
+      {
+      XmNkeyMode,                    /* Same as above */
+      XmCKeyMode,
+      XmRInt,
+      sizeof (int),
+      XtOffsetOf (AppRes, keyMode),
+      XtRImmediate,
+      (XtPointer) 0                 /* default to pulseaudio */
+      },
+
       {
       XmNcwSpeed,                   /* Same as above */
       XmCCwSpeed,
@@ -98,37 +105,17 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, cwSpeed),
       XtRImmediate,
-      (XtPointer) "10"
+      (XtPointer) 20
       },
 
       {
-      XmNcwLevel,                   /* Same as above */
-      XmCCwLevel,
+      XmNcwTone,                   /* Same as above */
+      XmCCwTone,
       XtRInt,
       sizeof (int),
-      XtOffsetOf (AppRes, cwLevel),
+      XtOffsetOf (AppRes, cwTone),
       XtRImmediate,
-      (XtPointer) "50"
-      },
-
-      {
-      XmNdspDev,                    /* Same as above */
-      XmCDspDev,
-      XtRString,
-      sizeof (String),
-      XtOffsetOf (AppRes, dspDev),
-      XtRImmediate,
-      (XtPointer) "/dev/dsp"
-      },
-
-      {
-      XmNmixerDev,                  /* Same as above */
-      XmCMixerDev,
-      XtRString,
-      sizeof (String),
-      XtOffsetOf (AppRes, mixerDev),
-      XtRImmediate,
-      (XtPointer) "/dev/mixer"
+      (XtPointer) 450
       },
 
       {
@@ -138,7 +125,7 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, autoReset),
       XtRImmediate,
-      (XtPointer)0
+      (XtPointer) 0
       },
 
       {
@@ -148,7 +135,7 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, autobeeps),
       XtRImmediate,
-      (XtPointer)5
+      (XtPointer) 5
       },
 
       {
@@ -158,7 +145,7 @@ int main (int argc, char *argv[])
       sizeof (Pixel),
       XtOffsetOf (AppRes, flashColor),
       XtRImmediate,
-      (XtPointer)1
+      (XtPointer) 1
       },
 
       {
@@ -168,7 +155,7 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, minutes),
       XtRImmediate,
-      (XtPointer)10
+      (XtPointer) 10
       },
 
       {
@@ -178,22 +165,31 @@ int main (int argc, char *argv[])
       sizeof (int),
       XtOffsetOf (AppRes, seconds),
       XtRImmediate,
-      (XtPointer)0
+      (XtPointer) 0
       },
   };
 
 #ifdef XmVERSION_STRING
    extern const char _XmVersionString[];
-   printf ("Compiled with %s\n", XmVERSION_STRING + 4);
-   printf ("Running  with %s\n", _XmVersionString + 4);
+   fprintf (stderr, "Compiled with %s\n", XmVERSION_STRING + 4);
+   fprintf (stderr, "Running  with %s\n", _XmVersionString + 4);
+
+   /* LessTif and Motif are counting buttons in popups differently
+    * Needs 10 for Motif and 5 for LessTif
+    */
+   if (strncmp(_XmVersionString +  4, "M", 1) == 0)
+      post_from = 10;
+   else
+      post_from = 5;
 #endif
 
    /*
     * Create clockShell and set some of its properties
     */
-
-   clock_shell = XtVaAppInitialize (&app, "Twclock",
-      NULL, 0, &argc, argv, NULL, 
+   clock_shell = XtVaOpenApplication (&app, "Twclock",
+      NULL, 0, &argc, argv, NULL,
+      sessionShellWidgetClass, 
+      XmNtitle, "TWCLOCK",
       XmNminHeight, 175,
       XmNminWidth, 175,
       XmNmaxHeight, 250,
@@ -202,23 +198,36 @@ int main (int argc, char *argv[])
       XmNminAspectY, 1,
       XmNmaxAspectX, 1,
       XmNmaxAspectY, 1,
-      XmNtitle, "TWCLOCK ",
-      /* XmNiconName, "TWCLOCK", */
       NULL);
 
    XtGetApplicationResources (clock_shell, &appRes, appRes_desc,
       XtNumber (appRes_desc), NULL, 0);
 
-   /* Initialize libCW */
-   cw_init(appRes.dspDev, appRes.mixerDev, NULL);
-   cw_set_wpm (appRes.cwSpeed);
-   cw_set_send_volume (appRes.cwLevel);
-   cw_set_key_modes (SC);
-
    /*
-    * If MAKE_ICON is defined, create its pixmap
+    * Check tzone to see if we are running on the default value (-1).
+    * If so, Twclock was not found and things won't look right.
+    */
+   if (appRes.tzone == -1)
+   {
+      fprintf (stderr, "twclock: The resource file, Twclock, not found\n");
+      XtRealizeWidget (clock_shell);   
+      errorDiag (clock_shell, "The resource file, Twclock, is missing.\n"
+                              "This is an installation problem.  Did you\n"
+                              "install twclock as root?  twclock must exit.\n\n"
+                              "Please send bug reports to\n" PACKAGE_BUGREPORT,
+                               CANCEL_BTN);
+      appRes.tzone = 1;               /* set time zone to GMT. or quit? */
+   }
+
+   /* Initialize libCW */
+   cw_init( appRes.keyMode, "TWCLOCK", "dummy");
+   cw_set_wpm (appRes.cwSpeed);
+   cw_set_keyMode (PA, "TWCLOCK");
+
+#if (MAKE_ICON == 1) && (HAVE_X11_XPM_H == 1) && (HAVE_LIBXPM == 1)
+   /*
+    *  then create pixmap for application Icon
     */
-#ifdef MAKE_ICON
    pix_attributes.closeness = 40000;
    pix_attributes.valuemask = XpmColorSymbols | XpmCloseness;
    pix_attributes.colorsymbols = transparentColor;
@@ -227,18 +236,18 @@ int main (int argc, char *argv[])
    XpmCreatePixmapFromData (XtDisplay(clock_shell),
       DefaultRootWindow (XtDisplay (clock_shell)),
       twclock_xpm, &pixmap, &mask, &pix_attributes);
+
+   XtVaSetValues (clock_shell,
+      XmNiconPixmap, pixmap,
+      NULL);
 #endif
+
    /*
     * Build and Realize the rest of the widget tree
     */
    build_widgets();
    XtRealizeWidget (clock_shell);   
 
-#ifdef MAKE_ICON
-   XtVaSetValues (clock_shell,
-      XmNiconPixmap, pixmap,
-      NULL);
-#endif
    /*
     * Enter the event loop
     */
