@@ -5,22 +5,11 @@
 #ifndef TWCLOCK_H_INCLUDED
 #define TWCLOCK_H_INCLUDED
 
-#define MAKE_ICON
-
-#ifdef MAKE_ICON
-#include <X11/xpm.h>
-#endif
-
 #include "libCW.h"
 #include <time.h> 
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include <sys/types.h>
-	
-#include <X11/Xlib.h>
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
 
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -39,21 +28,26 @@
 #include <Xm/RowColumn.h>
 #include <Xm/ScrolledW.h>
 #include <Xm/Separator.h>
+#include "../config.h"
+
+#define MAKE_ICON  1
 
 #define LOCAL_BTN  0
 #define GMT_BTN    1
 #define OTHERS_BTN 2
 #define SET_TIMER_BTN 3
 #define ID_NOW_BTN 4
-#define ABOUT_BTN 5
+#define HELP_BTN 5
 #define QRT 6
 
 #define ABOUT_DIAG  0
-#define HELP_DIAG    1
+#define HELP_DIAG   1
+#define CANCEL_BTN  1
+
 /*
  * Work around when Scale widget won't display both Value and Title
  */
-#define SCALE_BUG 1
+#define SCALE_BUG 0
 
 /*
  * APPLICATION DEFINED RESOURCES
@@ -73,17 +67,14 @@
 #define XmNcwStr           "cwStr"
 #define XmCCwStr           "CwStr"
 
+#define XmNkeyMode         "keyMode"
+#define XmCKeyMode         "KeyMode"
+
 #define XmNcwSpeed         "cwSpeed"
 #define XmCCwSpeed         "CwSpeed"
 
-#define XmNcwLevel         "cwLevel"
-#define XmCCwLevel         "CwLevel"
-
-#define XmNdspDev          "dspDev"
-#define XmCDspDev          "DspDev"
-
-#define XmNmixerDev        "mixerDev"
-#define XmCMixerDev        "MixerDev"
+#define XmNcwTone          "cwTone"
+#define XmCCwTone          "CwTone"
 
 #define XmNautoReset       "autoReset"
 #define XmCAutoReset       "AutoReset"
@@ -107,10 +98,9 @@ typedef struct
       int     beep;
       int     cwID;
       char    *cwStr;
+		int     keyMode;
       int     cwSpeed;
-      int     cwLevel;
-      char    *dspDev;
-      char    *mixerDev;
+      int     cwTone;
       int     autoReset;
       int     autobeeps;
       Pixel   flashColor;
@@ -124,6 +114,7 @@ typedef struct
 XtAppContext app;
 AppRes appRes;
 
+int    post_from;
 Widget clock_shell;
 Widget call_toggleB;
 Widget hr_scale;
@@ -142,7 +133,7 @@ XtIntervalId time_id;
 void build_widgets (void);
 void input_event (Widget, XtPointer, XEvent *, Boolean *);
 void popup_cb (Widget, XtPointer, XtPointer);
-void abt_cas_cb (Widget w, XtPointer client_data, XtPointer cbs);
+void help_cas_cb (Widget w, XtPointer client_data, XtPointer cbs);
 void popup_about (void);
 void popup_help (void);
 void TBchangeCB (Widget, XtPointer, XtPointer);
@@ -154,6 +145,9 @@ void fsbOkCB (Widget , XtPointer, XtPointer);
 void fsbCancelCB (Widget , XtPointer, XtPointer);
 void timerOkCB (Widget , XtPointer, XtPointer);
 void timerModVerCB (Widget , XtPointer, XtPointer);
+void errorDiag (Widget, char *, int);
+void errOkCB (Widget , XtPointer, XtPointer);
+void errCancelCB (Widget , XtPointer, XtPointer);
 void settimer (void);
 
 #endif
