@@ -1,6 +1,6 @@
 /*
  * twclockCB:  A world clock implemented with openMotif widgets
- * Copyright (C) 1997 - 2007 Ted Williams WA0EIR
+ * Copyright (C) 1997 - 2011 Ted Williams WA0EIR
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge. MA 02139, USA.
  * See the COPYING file in this directory
  *
- * Version: 2.7 - Aug 2007
+ * Version: 3.1 - Apr 2011
  */
 
 /*
@@ -30,6 +30,10 @@
 
 #include "twclock.h"
 
+#if (MAKE_ICON == 1) && (HAVE_X11_XPM_H == 1) && (HAVE_LIBXPM == 1)
+extern Pixmap pixmap;
+#endif
+
 Pixel saveColor;
 Boolean newTimeFlag;
 char timepath[100];
@@ -42,8 +46,9 @@ char timepath[100];
 void input_event (Widget w, XtPointer client_data, XEvent *event, Boolean *cont)
 {
    Widget popup_menu = (Widget) client_data;
-
-   if(event->type == ButtonPress && event->xbutton.button == Button3)
+   //if(event->type == ButtonPress && event->xbutton.button == Button3)
+   if (event->xbutton.button != Button3)
+      return;
    {
       XmMenuPosition(popup_menu, &(event->xbutton));
       XtManageChild(popup_menu); 
@@ -82,7 +87,10 @@ void popup_cb(Widget w, XtPointer client_data, XtPointer cbs)
             (ArgList) NULL, 0);
 
          XtVaSetValues (XtParent(zone_fsb),
-            XmNtitle, "Timezone Selection",
+            XmNtitle, "TWCLOCK TIME ZONE SELECTION",
+         #if (MAKE_ICON == 1) && (HAVE_X11_XPM_H == 1) && (HAVE_LIBXPM == 1)
+            XmNiconPixmap, pixmap,
+         #endif
             NULL);
 
          XtVaSetValues (zone_fsb,
@@ -116,18 +124,21 @@ void popup_cb(Widget w, XtPointer client_data, XtPointer cbs)
       case ID_NOW_BTN:
          cw_send_str (appRes.cwStr);
          break;
-#if 0
-      case ABOUT_BTN:
-         printf ("Hello World # %d\n", btn_num);
+
+      case HELP_BTN:
+         /* let the cascade callback do it */
          break;
-#endif
+
       case QRT:
          cleanup ();
          exit(0);
          break;
 
       default:
-         fprintf (stderr, "Error 1: Invalid menu button\n");
+         errorDiag (clock_shell, "Internal error\n"
+                                 "Invalid menu button\n",
+                                 !CANCEL_BTN);
+         fprintf (stderr, "twclock: Invalid menu button\n");
          break;
    }
 }
@@ -137,7 +148,7 @@ void popup_cb(Widget w, XtPointer client_data, XtPointer cbs)
  * callback for About Cascade button
  * switch on the button number
  */
-void abt_cas_cb(Widget w, XtPointer client_data, XtPointer cbs)
+void help_cas_cb(Widget w, XtPointer client_data, XtPointer cbs)
 {
    int btn = (int) client_data;
 
@@ -152,7 +163,10 @@ void abt_cas_cb(Widget w, XtPointer client_data, XtPointer cbs)
          break;
 
       default:
-         fprintf (stderr, "Error 2: Invalid button number");
+         errorDiag (clock_shell, "Internal error\n"
+                                 "Invalid button number\n",
+                                 !CANCEL_BTN);
+         fprintf (stderr, "twclock: Invalid button number");
          break;
    }
 }
@@ -195,7 +209,7 @@ void  gettime ()
    XtVaSetValues (sec_scale, XmNvalue, timestruct->tm_sec,  NULL);
    if ((timestruct->tm_mday != oldday) | doit)
    {
-      strftime (datestr, sizeof(datestr), "%e %b %Y %Z", timestruct);
+      strftime (datestr, sizeof(datestr), "%a %e %b %Y %Z", timestruct);
       labelstr = XmStringCreateLocalized (datestr);
 
       XtVaSetValues (date_label,
@@ -300,15 +314,20 @@ void TBchangeCB(Widget w, XtPointer cdata, XtPointer cbs)
          /* button is unset? This hack removes and adds a very */
          /* short time_id, which has no effect on the gui, but */
          /* keeps timer_id valid. */
+#if 0
          XtRemoveTimeOut (time_id);
          time_id=XtAppAddTimeOut(app,
             1L,
             (XtTimerCallbackProc)gettime,
             NULL);
+#endif
          break;
 
       default:
-         fprintf (stderr, "Error: invalid toggle button state\n");
+         errorDiag (clock_shell, "Internal error\n"
+                                 "Invalid toggle button state\n",
+                                 !CANCEL_BTN);
+         fprintf (stderr, "twclock: invalid toggle button state\n");
          break;
   }
 } 
@@ -316,8 +335,8 @@ void TBchangeCB(Widget w, XtPointer cdata, XtPointer cbs)
 /*
  * ALARM TIMER FUNCTION
  * First called when the ID alarm_timer times out.  It will cw id, blink the
- * toggle button and beep as requested in the resource file.  It then
- * changes the delay to 1 sec and registers another timeout.
+ * toggle button and beep as requested in the resource file/setup page.  It
+ * then changes the delay to 1 sec and registers another timeout.
  * Unsetting the toggle button unregisters the timeouts.
  */
 void alarm_timer (void)
@@ -367,17 +386,43 @@ void alarm_timer (void)
       cwIDdone = True;
    }
 
+   /* should we beep? */
    if (appRes.beep == True)
    {
-      if ((con = fopen ("/dev/console", "w")) == 0)
-      {
-         fprintf (stderr, "can't open console\n");
-         return;
-      }
-      fprintf (con, "\a");
-      fclose (con);
+      /*
+       * This code was stolen from 
+       * http://agendawiki.com/cgi-bin/aw.pl?UnusualInputOutput
+       * and modified to restore the system default values.
+       */
+      XKeyboardControl v;
+      XKeyboardState save;
+      Display *xdisplay = XOpenDisplay(NULL);
+
+      /* Get the current keyboard control values so we can restore them */
+      XGetKeyboardControl (xdisplay, &save);
+
+      /* Set the desired pitch, duration, and volume (percent) level */
+      v.bell_pitch = 800;
+      v.bell_duration = 100;
+      v.bell_percent = 100;
+      XChangeKeyboardControl
+         (xdisplay, KBBellPitch | KBBellDuration | KBBellPercent, &v);
+
+      /* Ding the bell and make sure it happens now */
+      XBell(xdisplay, 100);
+      XFlush(xdisplay);
+
+      /* Be nice and restore the original values */ 
+      v.bell_pitch = save.bell_pitch;
+      v.bell_duration = save.bell_duration;
+      v.bell_percent = save.bell_percent;
+      XChangeKeyboardControl
+         (xdisplay, KBBellPitch | KBBellDuration | KBBellPercent, &v);
+
+      XCloseDisplay(xdisplay);
    }
 
+   /* should we blink? */
    if (appRes.blink == True && flashFlag == True)
    {
       XtVaSetValues (call_toggleB,
@@ -413,9 +458,9 @@ void settimer (void)
    static Widget timeTFs[2];
    Widget label1, frame1, timeForm, min_label, sec_label;
    Widget label2, frame2, optForm, autoResetTB, cwidTB, beepTB, blinkTB;
-   Widget label3, frame3, cwRC, cwSpdScale, cwLvlScale;
+   Widget label3, frame3, cwRC, cwSpdScale, cwToneScale;
    Widget sep1, okPB, cancelPB;
-#if SCALE_BUG == 1
+#if SCALE_BUG == 0
    Widget label4, label5;
 #endif
    XmString xms;
@@ -427,31 +472,12 @@ void settimer (void)
    /* Lazy Instantiation */   
    if (settimerDiag != NULL)
    {
-#if 0
-     /* not needed with openMotif - maybe Lesstif does */
-     /* reload times in case we closed via Cancel button */
-     /* but got to remove the callbacks first */
-      XtRemoveCallback (timeTFs[0], XmNmodifyVerifyCallback, timerModVerCB,
-         (XtPointer) NULL);
-      XtRemoveCallback (timeTFs[1], XmNmodifyVerifyCallback, timerModVerCB,
-         (XtPointer) NULL);
-#endif
-
       XtVaSetValues (timeTFs[0],
          XmNvalue, minutes,
          NULL);
-
       XtVaSetValues (timeTFs[1],
          XmNvalue, seconds,
          NULL);
-#if 0
-      /* see above */
-      /* now add the callbacks again */
-      XtAddCallback (timeTFs[0], XmNmodifyVerifyCallback, timerModVerCB,
-         (XtPointer) NULL);
-      XtAddCallback (timeTFs[1], XmNmodifyVerifyCallback, timerModVerCB,
-         (XtPointer) NULL);
-#endif
       XtManageChild (settimerDiag);
       return;
    }
@@ -460,7 +486,10 @@ void settimer (void)
    settimerDiag = XmCreateFormDialog (clock_shell, "settimerDiag", NULL, 0);
 
    XtVaSetValues (XtParent(settimerDiag),
-      XmNtitle, "Twclock Options",
+      XmNtitle, "TWCLOCK OPTIONS",
+   #if (MAKE_ICON == 1) && (HAVE_X11_XPM_H == 1) && (HAVE_LIBXPM == 1)
+      XmNiconPixmap, pixmap,
+   #endif
       NULL); 
 
    XtVaSetValues (settimerDiag,          
@@ -643,44 +672,49 @@ void settimer (void)
 
    xms = XmStringCreateLocalized ("CW Speed");
    /* if we have the scale bug, put the sting in a label */
-#if SCALE_BUG == 1
-   label4 = XtVaCreateManagedWidget ("label3", xmLabelWidgetClass, cwRC,
-      XmNlabelString, xms,
-      NULL);
-#endif
+//#if SCALE_BUG == 1
+   //label4 = XtVaCreateManagedWidget ("label3", xmLabelWidgetClass, cwRC,
+      //XmNlabelString, xms,
+      //NULL);
+//#endif
 
-   cwSpdScale = XtVaCreateManagedWidget ("cwSptScale", xmScaleWidgetClass,
+   cwSpdScale = XtVaCreateManagedWidget ("cwSpdScale", xmScaleWidgetClass,
       cwRC,
       XmNscaleMultiple, 5,
       XmNorientation, XmHORIZONTAL,
       XmNshowValue, True,
       XmNminimum, 10,
       XmNmaximum, 40,
+      XmNvalue, appRes.cwSpeed,
+#if SCALE_BUG == 0
       XmNtitleString, xms,
+#endif
       NULL);
-
    /* free the string and sync cwSpdScale with cwSpeed resource */
    XmStringFree(xms);
-   XmScaleSetValue (cwSpdScale, appRes.cwSpeed);
 
-   xms = XmStringCreateLocalized ("CW Level");
+   xms = XmStringCreateLocalized ("CW Tone");
    /* if we have the scale bug, put the sting in a label */
-#if SCALE_BUG == 1
-   label5 = XtVaCreateManagedWidget ("label3", xmLabelWidgetClass, cwRC,
-      XmNlabelString, xms,
-      NULL);
-#endif
+//#if SCALE_BUG == 1
+   //label5 = XtVaCreateManagedWidget ("label5", xmLabelWidgetClass, cwRC,
+      //XmNlabelString, xms,
+      //NULL);
+//#endif
 
-   cwLvlScale = XtVaCreateManagedWidget ("cwLvlScale", xmScaleWidgetClass,
+   cwToneScale = XtVaCreateManagedWidget ("cwToneScale", xmScaleWidgetClass,
       cwRC,
       XmNorientation, XmHORIZONTAL,
+		XmNscaleMultiple, 10,
       XmNshowValue, True,
+      XmNminimum, 500,
+		XmNmaximum, 1300,
+      XmNvalue, appRes.cwTone,
+#if SCALE_BUG == 0
       XmNtitleString, xms,
+#endif
       NULL);
-
-   /* free the string and sync cwLvlScale with cwLevel resource */
+   /* free the string and sync cwToneScale with cwLevel resource */
    XmStringFree(xms);
-   XmScaleSetValue (cwLvlScale, appRes.cwLevel);
 
    sep1 = XtVaCreateManagedWidget ("sep1", xmSeparatorWidgetClass, settimerDiag,
       XmNtopAttachment, XmATTACH_WIDGET,
@@ -741,15 +775,14 @@ void settimer (void)
     */
    XtAddCallback (cwSpdScale, XmNdragCallback, scalesCB,
                (XtPointer) 0);
-               //(XtPointer) &appRes.cwSpeed);
 
    XtAddCallback (cwSpdScale, XmNvalueChangedCallback, scalesCB,
                (XtPointer) 0);
 
-   XtAddCallback (cwLvlScale, XmNdragCallback, scalesCB,
+   XtAddCallback (cwToneScale, XmNdragCallback, scalesCB,
                (XtPointer) 1);
 
-   XtAddCallback (cwLvlScale, XmNvalueChangedCallback, scalesCB,
+   XtAddCallback (cwToneScale, XmNvalueChangedCallback, scalesCB,
                (XtPointer) 1);
 
    /* Manage all the dialog kids */
@@ -819,7 +852,7 @@ void timerModVerCB (Widget w, XtPointer client_data, XtPointer cbs)
 
    for (i=0; i < pt->text->length; i++)
    {
-      if (isdigit (pt->text->ptr[i]))  /* allow only digits, beep if not */
+      if (isdigit (pt->text->ptr[i]))  /* allow only digits */
       {
          newTimeFlag = True;
          return;
@@ -850,8 +883,7 @@ void optionsCB (Widget w, XtPointer client_data, XtPointer cbs)
 
 /*
  * scalesCB drag/valueChanged callbacks for CW ID scales
- * maintains the values of CW ID option resources
- * client_data is is stored
+ * client_data is 0 for speed scale and 1 for tone
  */
 void scalesCB (Widget w, XtPointer client_data, XtPointer cbs)
 {
@@ -869,11 +901,14 @@ void scalesCB (Widget w, XtPointer client_data, XtPointer cbs)
          break;
 
       case 1:
-         appRes.cwLevel = val;
-         cw_set_send_volume (appRes.cwLevel);  
+         appRes.cwTone = val;
+         cw_set_freq (appRes.cwTone);  
          break;
 
       default:
-         fprintf (stderr, "scalesCB: invalid argument\n");
+         errorDiag (clock_shell, "Internal error\n"
+                                 "Invalid scalesCB argument\n",
+                                 !CANCEL_BTN);
+         fprintf (stderr, "twclock: invalid argument\n");
    }
 }
