@@ -1,6 +1,6 @@
 /*
  * twclockWids:  A world clock implemented with openMotif widgets
- * Copyright (C) 1997 - 2007 Ted Williams WA0EIR 
+ * Copyright (C) 1997 - 2011 Ted Williams WA0EIR 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge. MA 02139, USA.
  * See COPYING file in this directory.
  *
- * Version: 2.7 - Oct 2007
+ * Version: 3.1 - Apr 2011
  */
 
 /*
@@ -28,7 +28,7 @@
 
 void  build_widgets ()
 {
-   Widget form, separator, popup_menu, about_menu;
+   Widget form, separator, popup_menu, help_cas_menu;
 
    XmString xms[20];
    int i,j;
@@ -79,8 +79,8 @@ void  build_widgets ()
       XmNrightAttachment, XmATTACH_FORM,
       XmNleftAttachment, XmATTACH_FORM,
       XmNtopAttachment, XmATTACH_FORM,
-      XmNhighlightThickness, 0,   /* these make it look like a push on/push off button */
-      XmNshadowThickness, 3,
+      XmNhighlightThickness, 0,   /* these make it look like  */
+      XmNshadowThickness, 3,      /* a push on/push off button */
       XmNindicatorOn, XmINDICATOR_NONE,
       XmNfillOnSelect, False,
       NULL);
@@ -99,6 +99,7 @@ void  build_widgets ()
       XmNtopAttachment, XmATTACH_WIDGET,
       XmNtopWidget, call_toggleB,
       XmNmaximum, 23,
+      XmNeditable, False,
       XmNshowValue, True,
       NULL);
 
@@ -119,6 +120,7 @@ void  build_widgets ()
       XmNtopAttachment, XmATTACH_WIDGET,
       XmNtopWidget, call_toggleB,
       XmNmaximum, 59,
+      XmNeditable, False,
       XmNshowValue, True,
       NULL);
 
@@ -136,6 +138,7 @@ void  build_widgets ()
       XmNtopAttachment, XmATTACH_WIDGET,
       XmNtopWidget, call_toggleB,
       XmNmaximum, 59,
+      XmNeditable, False,
       XmNshowValue, True,
       NULL);
 
@@ -158,7 +161,7 @@ void  build_widgets ()
    xms[i] = XmStringCreateLocalized ("Ctrl-S"); i++;
    xms[i] = XmStringCreateLocalized ("ID Now"); i++;
    xms[i] = XmStringCreateLocalized ("Ctrl-I"); i++;
-   xms[i] = XmStringCreateLocalized ("About"); i++;
+   xms[i] = XmStringCreateLocalized ("Help"); i++;
    xms[i] = XmStringCreateLocalized ("QRT"); i++;
    xms[i] = XmStringCreateLocalized ("Ctrl-Q"); i++;
 
@@ -177,7 +180,7 @@ void  build_widgets ()
       XmVaSEPARATOR,
       XmVaPUSHBUTTON, xms[9], 'I', "Ctrl<Key>I", xms[10],
       XmVaSEPARATOR,
-      XmVaCASCADEBUTTON, xms[11], 'A',
+      XmVaCASCADEBUTTON, xms[11], NULL,
       XmVaSEPARATOR,
       XmVaPUSHBUTTON, xms[12], 'Q', "Ctrl<Key>Q", xms[13],
       NULL);
@@ -191,16 +194,16 @@ void  build_widgets ()
    }
 
    /*
-    * Create pulldown menu for the About cascade button 
+    * Create cascade menu for the Help cascade button 
     */
    i = 0;
    xms[i] = XmStringCreateLocalized ("About"); i++;
    xms[i] = XmStringCreateLocalized ("Help"); i++;
 
-   about_menu = XmVaCreateSimplePulldownMenu (popup_menu, "about_menu", 10,
-      abt_cas_cb,
-      XmVaPUSHBUTTON, xms[0], 'P', "Ctrl<Key>P", NULL,
-      XmVaPUSHBUTTON, xms[1], 'N', "Ctrl<Key>N", NULL,
+   help_cas_menu = XmVaCreateSimplePulldownMenu (popup_menu,"help_cas_menu",
+      post_from, help_cas_cb,
+      XmVaPUSHBUTTON, xms[0], NULL, "", NULL,
+      XmVaPUSHBUTTON, xms[1], NULL, "", NULL,
       NULL);
 
    /*
@@ -218,22 +221,34 @@ void  build_widgets ()
  * to the start up time zone.  Then gettime() sets TZ and initializes
  * the scales.
  */
-   XtAddEventHandler (form,ButtonPressMask,False,input_event,popup_menu);
+   XtAddEventHandler (form ,ButtonPressMask, False, input_event, popup_menu);
 
-   /*
-    * This is a hack to make the right mouse event be recognized on all the
-    * children.  Well, all but the slider control in the scale
-    */
+/*
+* This is a hack to make the right mouse event be recognized on all the
+* children, but the slider in the scale didn't work.  Now they work too
+* with the addition of the last three XtAddEventHandler
+*/
 #ifdef LESSTIF_VERSION 
-   printf ("doing hack for lesstif\n");
+//#if 0
+/* mouse grab bug - force lesstif fix for xorg bug */
+/* control keys still won't work cuz no menu bar */
+   fprintf (stderr, "doing hack for lesstif\n");
    XtAddEventHandler (date_label,ButtonPressMask,False,input_event,popup_menu);
    XtAddEventHandler (separator,ButtonPressMask,False,input_event,popup_menu);
-   XtAddEventHandler (call_toggleB,ButtonPressMask,False,input_event,popup_menu);
+   XtAddEventHandler (call_toggleB,ButtonPressMask,False,input_event,
+	popup_menu);
    XtAddEventHandler (hr_scale,ButtonPressMask,False,input_event,popup_menu);
    XtAddEventHandler (min_scale,ButtonPressMask,False,input_event,popup_menu);
    XtAddEventHandler (sec_scale,ButtonPressMask,False,input_event,popup_menu);
+   XtAddEventHandler (XtNameToWidget(hr_scale,"Scrollbar"),ButtonPressMask,
+                      False,input_event,popup_menu);
+   XtAddEventHandler (XtNameToWidget(min_scale,"Scrollbar"),ButtonPressMask,
+                      False,input_event,popup_menu);
+   XtAddEventHandler (XtNameToWidget(sec_scale,"Scrollbar"),ButtonPressMask,
+                      False,input_event,popup_menu);
 #endif
 
+   /* call the popup_cb to set the time to start up xdb value. */
    popup_cb (form, (XtPointer)(appRes.tzone), (XtPointer)NULL);
    gettime ();
 
